/*
 * IRx8.c
 *
 * Created: 2014-12-29 14:47:30
 *  Author: dsosnowski
 */ 


//includes
#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include "ui.h"
#include "defs.h"
#include "scanner.h"
//#include "ee_mem.h"
//#include <avr/pgmspace.h>


//---------------------------------------------------------------------------------
void HardInit(void);
void PeriodTimerInit(void);
void PeriodProcess(void);
void WdtSet(void);
void WdtReset(void);

struct {
	char idle;
	char prog;
	char action;
	char irdelay;
	char configmask;
	char cleardelay;
	char newcode;
} system;



//---------------------------------------------------------------------------------
int main(void)
{

	WdtSet();
	HardInit();

	_delay_ms(100);

	UiLedBlink(LONG_BLINK, LONG_BLINK);
	PeriodTimerInit();
	ScannerInit();
	system.irdelay = LONG_IR_DELAY;
    
	while(1)
    {
		
		//---normal mode------------------------------------
		if (system.prog == 0)
		{
			//---------------------------------------
			if (ScannerStatus() == SCAN_COMPLETE)
			{
				UiLedBlink(0,2);
				system.idle = IDLE_DELAY;
				
				system.action = 0;
				system.action = ScannerComp();
				if (system.action)
				{
					UiLedBlink(20,0);
					system.action += 1;
					system.action /= 2;
					system.irdelay = DEFAULT_IR_DELAY;
				}
				else	
					system.irdelay = 2;
			}
			
			//---------------------------------------
			if ((system.action) && (system.action <= 8))
			{				
				if (system.action != system.newcode)
				{
					UiOutputClear(UiConfigGet());
					UiOutputToggle(1<<(system.action - 1));
					system.cleardelay =  DEFAULT_CLEAR_DELAY;
				}
				else
				{
					system.cleardelay =  DEFAULT_CLEAR_DELAY;				
				}

				system.newcode = system.action;
				system.action = 0;
			}
			else
				system.action = 0;


			//---------------------------------------
			if (system.cleardelay)
			{
				system.cleardelay--;
				if (system.cleardelay == 0)
				{
					UiOutputClear(UiConfigGet());
					system.newcode = 0;
				}
			}

			//---------------------------------------
			if (UiProgGet())
			{
				ScannerStop();
				system.prog = 1;
			}			
		}
		
		//---program mode------------------------------------
		else
		{
			if (system.prog == 1)
			{
				system.action = UiChannelGet();
				
				if (system.action)
					UiOutputState(1<<(system.action - 1));
				else
					UiOutputState(0);
				
				if (UiProgGet() == 0) system.prog++;

				UiLedBlink(LONG_BLINK,LONG_BLINK);
				system.irdelay = LONG_IR_DELAY;
			}
			
			if (system.prog == 2)
			{
				UiLedBlink(0,LONG_BLINK);			
			}
			
			if (system.prog == 3)
			{
				UiLedBlink(LONG_BLINK,0);
			}

			if (system.prog <= 3)
			{
				if (ScannerStatus() == SCAN_COMPLETE)
				{
					if (system.prog == 2)
					ScannerSave((system.action * 2) - 1);

					if (system.prog == 3)
					ScannerSave((system.action * 2));
						
					system.irdelay = LONG_IR_DELAY;
					system.prog++;
				}
			}

			if (system.prog == 4)
			{
				UiLedBlink(LONG_BLINK,LONG_BLINK);
				if (system.irdelay == 0) system.prog = 5;
			}

			if (system.prog > 4)
			{
				UiOutputState(0);
				UiLedBlink(LONG_BLINK,LONG_BLINK);
				system.idle = IDLE_DELAY;
				system.action = 0;
				system.prog = 0;
			}
		}

		//---------------------------------------
		if (system.irdelay)
		{
			system.irdelay--;
			if (system.irdelay == 0) ScannerStart();
		}
		
		//---------------------------------------
		system.idle--;
		if (system.idle == 0)
		{
			system.idle = IDLE_DELAY;
			UiLedBlink(0,2);
		}
		
		//---------------------------------------
		UiLedBlink(0,0);
		PeriodProcess();
		WdtReset();
    }
}

//---------------------------------------------------------------------------------
void HardInit(void)
{
	UI_PORT |= (1<<PROG_PIN | 1<<LED1_PIN | 1<<LED2_PIN);
	UI_DDR |= (1<<LED1_PIN | 1<<LED2_PIN);
	
	IR_PORT |= (1<<IR1_PIN);

	OUTPUT13_DDR |= (1<<OUT1_PIN | 1<<OUT2_PIN | 1<<OUT3_PIN);
	OUTPUT48_DDR |= (1<<OUT4_PIN | 1<<OUT5_PIN | 1<<OUT6_PIN | 1<<OUT7_PIN | 1<<OUT8_PIN);
	
	ACSR |= (1<<ACD);
	PRR = (1<<PRTWI | 1<<PRTIM2 | 1<<PRSPI | 1<<PRUSART0 | 1<<PRADC);
}

//---------------------------------------------------------------------------------
void PeriodTimerInit(void)
{
	TCCR1A = (1<<WGM11);
	TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS11);
	ICR1 = (MAIN_DELAY * 1000);
	TIFR1 = (1<<TOV1);
}

//---------------------------------------------------------------------------------
void PeriodProcess(void)
{
	while ((TIFR1 & (1<<TOV1)) == 0);
	TIFR1 = (1<<TOV1);	
}

//----------------------------------------
void WdtSet(void)
{
	wdt_reset();
	WDTCSR |= (1<<WDCE | 1<<WDE);
	WDTCSR = (1<<WDE | 7);
}

//----------------------------------------
void WdtReset(void)
{
	wdt_reset();
}
