/*
 * ui.c
 *
 * Created: 2014-12-31 09:15:53
 *  Author: dsosnowski
 */ 

//includes
#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include "ui.h"
#include "defs.h"
//#include "scanner.h"
//#include "ee_mem.h"
//#include <avr/pgmspace.h>



//---------------------------------------------------------------------------------
char UiConfigGet(void)
{
	char result = 0;
	
	CONFIG17_PORT |= (1<<CF1_PIN | 1<<CF2_PIN | 1<<CF5_PIN | 1<<CF6_PIN | 1<<CF7_PIN);
	CONFIG348_PORT |= (1<<CF3_PIN | 1<<CF4_PIN | 1<<CF8_PIN);
	_delay_ms(4);

	if ((CONFIG17_PIN & (1<<CF2_PIN)) == 0) result |= 1; 	
	if ((CONFIG17_PIN & (1<<CF1_PIN)) == 0) result |= 2;
	if ((CONFIG348_PIN & (1<<CF3_PIN)) == 0) result |= 4;
	if ((CONFIG348_PIN & (1<<CF4_PIN)) == 0) result |= 8;
	if ((CONFIG17_PIN & (1<<CF5_PIN)) == 0) result |= 16;
	if ((CONFIG17_PIN & (1<<CF6_PIN)) == 0) result |= 32;
	if ((CONFIG17_PIN & (1<<CF7_PIN)) == 0) result |= 64;
	if ((CONFIG348_PIN & (1<<CF8_PIN)) == 0) result |= 128;

	CONFIG17_PORT &= ~(1<<CF1_PIN | 1<<CF2_PIN | 1<<CF5_PIN | 1<<CF6_PIN | 1<<CF7_PIN);
	CONFIG348_PORT &= ~(1<<CF3_PIN | 1<<CF4_PIN | 1<<CF8_PIN);
	return result;
}
//---------------------------------------------------------------------------------
char UiChannelGet(void)
{
	char result = 0;
	
	CONFIG17_PORT |= (1<<CF1_PIN | 1<<CF2_PIN | 1<<CF5_PIN | 1<<CF6_PIN | 1<<CF7_PIN);
	CONFIG348_PORT |= (1<<CF3_PIN | 1<<CF4_PIN | 1<<CF8_PIN);
	_delay_ms(4);

	if ((CONFIG17_PIN & (1<<CF2_PIN)) == 0) result = 1;
	if ((CONFIG17_PIN & (1<<CF1_PIN)) == 0) result = 2;
	if ((CONFIG348_PIN & (1<<CF3_PIN)) == 0) result = 3;
	if ((CONFIG348_PIN & (1<<CF4_PIN)) == 0) result = 4;
	if ((CONFIG17_PIN & (1<<CF5_PIN)) == 0) result = 5;
	if ((CONFIG17_PIN & (1<<CF6_PIN)) == 0) result = 6;
	if ((CONFIG17_PIN & (1<<CF7_PIN)) == 0) result = 7;	
	if ((CONFIG348_PIN & (1<<CF8_PIN)) == 0) result = 8;

	CONFIG17_PORT &= ~(1<<CF1_PIN | 1<<CF2_PIN | 1<<CF5_PIN | 1<<CF6_PIN | 1<<CF7_PIN);
	CONFIG348_PORT &= ~(1<<CF3_PIN | 1<<CF4_PIN | 1<<CF8_PIN);
	return result;
}


//---------------------------------------------------------------------------------
void UiOutputState(char state)
{
	if (state & 1) OUTPUT48_PORT |= (1<<OUT8_PIN);
	else OUTPUT48_PORT &= ~(1<<OUT8_PIN);

	if (state & 2) OUTPUT48_PORT |= (1<<OUT7_PIN);
	else OUTPUT48_PORT &= ~(1<<OUT7_PIN);

	if (state & 4) OUTPUT48_PORT |= (1<<OUT6_PIN);
	else OUTPUT48_PORT &= ~(1<<OUT6_PIN);

	if (state & 8) OUTPUT48_PORT |= (1<<OUT5_PIN);
	else OUTPUT48_PORT &= ~(1<<OUT5_PIN);

	if (state & 16) OUTPUT48_PORT |= (1<<OUT4_PIN);
	else OUTPUT48_PORT &= ~(1<<OUT4_PIN);

	if (state & 32) OUTPUT13_PORT |= (1<<OUT3_PIN);
	else OUTPUT13_PORT &= ~(1<<OUT3_PIN);

	if (state & 64) OUTPUT13_PORT |= (1<<OUT2_PIN);
	else OUTPUT13_PORT &= ~(1<<OUT2_PIN);

	if (state & 128) OUTPUT13_PORT |= (1<<OUT1_PIN);
	else OUTPUT13_PORT &= ~(1<<OUT1_PIN);
}

//---------------------------------------------------------------------------------
void UiOutputToggle(char mask)
{
	if (mask & 1)
	{
		if (OUTPUT48_PIN & (1<<OUT8_PIN)) 
		OUTPUT48_PORT &= ~(1<<OUT8_PIN);
		else  OUTPUT48_PORT |= (1<<OUT8_PIN);
	}
	if (mask & 2)
	{
		if (OUTPUT48_PIN & (1<<OUT7_PIN))
		OUTPUT48_PORT &= ~(1<<OUT7_PIN);
		else  OUTPUT48_PORT |= (1<<OUT7_PIN);
	}
	if (mask & 4)
	{
		if (OUTPUT48_PIN & (1<<OUT6_PIN))
		OUTPUT48_PORT &= ~(1<<OUT6_PIN);
		else  OUTPUT48_PORT |= (1<<OUT6_PIN);
	}
	if (mask & 8)
	{
		if (OUTPUT48_PIN & (1<<OUT5_PIN))
		OUTPUT48_PORT &= ~(1<<OUT5_PIN);
		else  OUTPUT48_PORT |= (1<<OUT5_PIN);
	}
	if (mask & 16)
	{
		if (OUTPUT48_PIN & (1<<OUT4_PIN))
		OUTPUT48_PORT &= ~(1<<OUT4_PIN);
		else  OUTPUT48_PORT |= (1<<OUT4_PIN);
	}
	if (mask & 32)
	{
		if (OUTPUT13_PIN & (1<<OUT3_PIN))
		OUTPUT13_PORT &= ~(1<<OUT3_PIN);
		else  OUTPUT13_PORT |= (1<<OUT3_PIN);
	}
	if (mask & 64)
	{
		if (OUTPUT13_PIN & (1<<OUT2_PIN))
		OUTPUT13_PORT &= ~(1<<OUT2_PIN);
		else  OUTPUT13_PORT |= (1<<OUT2_PIN);
	}
	if (mask & 128)
	{
		if (OUTPUT13_PIN & (1<<OUT1_PIN))
		OUTPUT13_PORT &= ~(1<<OUT1_PIN);
		else  OUTPUT13_PORT |= (1<<OUT1_PIN);
	}
}

//---------------------------------------------------------------------------------
void UiOutputClear(char mask)
{
	if (mask & 1) OUTPUT48_PORT &= ~(1<<OUT8_PIN);

	if (mask & 2) OUTPUT48_PORT &= ~(1<<OUT7_PIN);

	if (mask & 4) OUTPUT48_PORT &= ~(1<<OUT6_PIN);

	if (mask & 8) OUTPUT48_PORT &= ~(1<<OUT5_PIN);

	if (mask & 16) OUTPUT48_PORT &= ~(1<<OUT4_PIN);

	if (mask & 32) OUTPUT13_PORT &= ~(1<<OUT3_PIN);

	if (mask & 64) OUTPUT13_PORT &= ~(1<<OUT2_PIN);

	if (mask & 128) OUTPUT13_PORT &= ~(1<<OUT1_PIN);
}


//---------------------------------------------------------------------------------
char UiProgGet(void)
{
	static char state;
	char result = 0;
	
	if (PROG_GET == 0)
	{
		if (state < 0xFF) state++;
		if (state == PROG_THRESHOLD) result = PROG_ENTER;
		if (state > PROG_THRESHOLD) result = PROG_CONTINUE;		
	}
	else
	{
		state = 0;
	}
		
	return result;
}

//---------------------------------------------------------------------------------
void UiLedBlink(char led_st, char led_ir)
{
	static char blink1;
	static char blink2;

	//update blinks
	if (led_st) blink1 = led_st;
	if (led_ir) blink2 = led_ir;
	
	//countdown process
	if ((led_st == 0) && (led_ir ==0 ))
	{
		if (blink1)
		{
			blink1--;
			if (blink1 == 0) LED_ST_OFF;
			else LED_ST_ON;
		}
		if (blink2)
		{
			blink2--;
			if (blink2 == 0) LED_IR_OFF;
			else LED_IR_ON;
		}
	}
}